--
-- BackHoe Digger Working Script
--
-- Original by Zippo
-- Modified with permission by Snapper 10/11/2011
--
-- Please Do not redistribute (modified or unmodified) without permission
--


BackHoeDigger = {}

function BackHoeDigger.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Steerable, specializations);
end;

function BackHoeDigger:load(xmlFile)
	
    -- Moving Parts
    local transBackActorNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.transBackActor#index"));
    if transBackActorNode ~= nil then
        self.transBackActor = {};
        self.transBackActor.node = transBackActorNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.transBackActor#minTrans"));
        self.transBackActor.minTrans = {};
        self.transBackActor.minTrans[1] = Utils.getNoNil(x, 0);
        self.transBackActor.minTrans[2] = Utils.getNoNil(y, 0);
        self.transBackActor.minTrans[3] = Utils.getNoNil(z, 0);

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.transBackActor#maxTrans"));
        self.transBackActor.maxTrans = {};
        self.transBackActor.maxTrans[1] = Utils.getNoNil(x, 0);
        self.transBackActor.maxTrans[2] = Utils.getNoNil(y, 0);
        self.transBackActor.maxTrans[3] = Utils.getNoNil(z, 0);

        self.transBackActor.transTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.transBackActor#transTime"), 2)*1000;
        self.transBackActor.touchTransLimit = Utils.getNoNil(getXMLString(xmlFile, "vehicle.transBackActor#touchTransLimit"), 10);
    end;

    local transBackActorBucketRamPinNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.transBackActorBucketRamPin#index"));
    if transBackActorBucketRamPinNode ~= nil then
        self.transBackActorBucketRamPin = {};
        self.transBackActorBucketRamPin.node = transBackActorBucketRamPinNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.transBackActorBucketRamPin#minTrans"));
        self.transBackActorBucketRamPin.minTrans = {};
        self.transBackActorBucketRamPin.minTrans[1] = Utils.getNoNil(x, 0);
        self.transBackActorBucketRamPin.minTrans[2] = Utils.getNoNil(y, 0);
        self.transBackActorBucketRamPin.minTrans[3] = Utils.getNoNil(z, 0);

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.transBackActorBucketRamPin#maxTrans"));
        self.transBackActorBucketRamPin.maxTrans = {};
        self.transBackActorBucketRamPin.maxTrans[1] = Utils.getNoNil(x, 0);
        self.transBackActorBucketRamPin.maxTrans[2] = Utils.getNoNil(y, 0);
        self.transBackActorBucketRamPin.maxTrans[3] = Utils.getNoNil(z, 0);

        self.transBackActorBucketRamPin.transTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.transBackActorBucketRamPin#transTime"), 2)*1000;
        self.transBackActorBucketRamPin.touchTransLimit = Utils.getNoNil(getXMLString(xmlFile, "vehicle.transBackActorBucketRamPin#touchTransLimit"), 10);
    end;

    local transBackActorTelescopicNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.transBackActorTelescopic#index"));
    if transBackActorTelescopicNode ~= nil then
        self.transBackActorTelescopic = {};
        self.transBackActorTelescopic.node = transBackActorTelescopicNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.transBackActorTelescopic#minTrans"));
        self.transBackActorTelescopic.minTrans = {};
        self.transBackActorTelescopic.minTrans[1] = Utils.getNoNil(x, 0);
        self.transBackActorTelescopic.minTrans[2] = Utils.getNoNil(y, 0);
        self.transBackActorTelescopic.minTrans[3] = Utils.getNoNil(z, 0);

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.transBackActorTelescopic#maxTrans"));
        self.transBackActorTelescopic.maxTrans = {};
        self.transBackActorTelescopic.maxTrans[1] = Utils.getNoNil(x, 0);
        self.transBackActorTelescopic.maxTrans[2] = Utils.getNoNil(y, 0);
        self.transBackActorTelescopic.maxTrans[3] = Utils.getNoNil(z, 0);

        self.transBackActorTelescopic.transTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.transBackActorTelescopic#transTime"), 2)*1000;
        self.transBackActorTelescopic.touchTransLimit = Utils.getNoNil(getXMLString(xmlFile, "vehicle.transBackActorTelescopic#touchTransLimit"), 10);
    end;

   local rotBackActorNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.rotBackActor#index"));
   if rotBackActorNode ~= nil then
        self.rotBackActor = {};
        self.rotBackActor.node = rotBackActorNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotBackActor#minRot"));
        self.rotBackActor.minRot = {};
        self.rotBackActor.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotBackActor.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotBackActor.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotBackActor#maxRot"));
        self.rotBackActor.maxRot = {};
        self.rotBackActor.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotBackActor.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotBackActor.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.rotBackActor.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotBackActor#rotTime"), 2)*1000;
        self.rotBackActor.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotBackActor#touchRotLimit"), 10));
    end;
	
    local rotBackActorBoomNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.rotBackActorBoom#index"));
    if rotBackActorBoomNode ~= nil then
        self.rotBackActorBoom = {};
        self.rotBackActorBoom.node = rotBackActorBoomNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotBackActorBoom#minRot"));
        self.rotBackActorBoom.minRot = {};
        self.rotBackActorBoom.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotBackActorBoom.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotBackActorBoom.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotBackActorBoom#maxRot"));
        self.rotBackActorBoom.maxRot = {};
        self.rotBackActorBoom.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotBackActorBoom.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotBackActorBoom.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.rotBackActorBoom.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotBackActorBoom#rotTime"), 2)*1000;
        self.rotBackActorBoom.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotBackActorBoom#touchRotLimit"), 10));
    end;
    
    local boomRamAnimRootNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.backActorBoomRam#index"));
    self.boomRamAnimCharSet = 0;
    if boomRamAnimRootNode ~= nil and boomRamAnimRootNode ~= 0 then
        self.boomRamAnimCharSet = getAnimCharacterSet(boomRamAnimRootNode);
        if self.boomRamAnimCharSet ~= 0 then
            local clip = getAnimClipIndex(self.boomRamAnimCharSet, getXMLString(xmlFile, "vehicle.backActorBoomRam#clip"));
            assignAnimTrackClip(self.boomRamAnimCharSet, 0, clip);
            setAnimTrackLoopState(self.boomRamAnimCharSet, 0, false);
            self.boomRamAnimSpeed = 0;
            setAnimTrackSpeedScale(self.boomRamAnimCharSet, 0, self.boomRamAnimSpeed);
            self.boomRamAnimDuration = getAnimClipDuration(self.boomRamAnimCharSet, clip);
            enableAnimTrack(self.boomRamAnimCharSet, 0);
        end;
    end;

    local rotBackActorDipperNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.rotBackActorDipper#index"));
        if rotBackActorDipperNode ~= nil then
        self.rotBackActorDipper = {};
        self.rotBackActorDipper.node = rotBackActorDipperNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotBackActorDipper#minRot"));
        self.rotBackActorDipper.minRot = {};
        self.rotBackActorDipper.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotBackActorDipper.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotBackActorDipper.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotBackActorDipper#maxRot"));
        self.rotBackActorDipper.maxRot = {};
        self.rotBackActorDipper.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotBackActorDipper.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotBackActorDipper.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.rotBackActorDipper.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotBackActorDipper#rotTime"), 2)*1000;
        self.rotBackActorDipper.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotBackActorDipper#touchRotLimit"), 10));
    end;
    
    local dipperRamAnimRootNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.backActorDipperRam#index"));
    self.dipperRamAnimCharSet = 0;
    if dipperRamAnimRootNode ~= nil and dipperRamAnimRootNode ~= 0 then
        self.dipperRamAnimCharSet = getAnimCharacterSet(dipperRamAnimRootNode);
        if self.dipperRamAnimCharSet ~= 0 then
            local clip = getAnimClipIndex(self.dipperRamAnimCharSet, getXMLString(xmlFile, "vehicle.backActorDipperRam#clip"));
            assignAnimTrackClip(self.dipperRamAnimCharSet, 0, clip);
            setAnimTrackLoopState(self.dipperRamAnimCharSet, 0, false);
            self.dipperRamAnimSpeed = 0;
            setAnimTrackSpeedScale(self.dipperRamAnimCharSet, 0, self.dipperRamAnimSpeed);
            self.dipperRamAnimDuration = getAnimClipDuration(self.dipperRamAnimCharSet, clip);
            enableAnimTrack(self.dipperRamAnimCharSet, 0);
        end;
    end;

    local rotBackActorBucketQHitchNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.rotBackActorBucketQHitch#index"));
        if rotBackActorBucketQHitchNode ~= nil then
        self.rotBackActorBucketQHitch = {};
        self.rotBackActorBucketQHitch.node = rotBackActorBucketQHitchNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotBackActorBucketQHitch#minRot"));
        self.rotBackActorBucketQHitch.minRot = {};
        self.rotBackActorBucketQHitch.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotBackActorBucketQHitch.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotBackActorBucketQHitch.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotBackActorBucketQHitch#maxRot"));
        self.rotBackActorBucketQHitch.maxRot = {};
        self.rotBackActorBucketQHitch.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotBackActorBucketQHitch.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotBackActorBucketQHitch.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.rotBackActorBucketQHitch.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotBackActorBucketQHitch#rotTime"), 2)*1000;
        self.rotBackActorBucketQHitch.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotBackActorBucketQHitch#touchRotLimit"), 10));
    end;
 
      
    local rotBackActorBucketNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.rotBackActorBucket#index"));
        if rotBackActorBucketNode ~= nil then
        self.rotBackActorBucket = {};
        self.rotBackActorBucket.node = rotBackActorBucketNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotBackActorBucket#minRot"));
        self.rotBackActorBucket.minRot = {};
        self.rotBackActorBucket.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotBackActorBucket.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotBackActorBucket.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotBackActorBucket#maxRot"));
        self.rotBackActorBucket.maxRot = {};
        self.rotBackActorBucket.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotBackActorBucket.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotBackActorBucket.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.rotBackActorBucket.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotBackActorBucket#rotTime"), 2)*1000;
        self.rotBackActorBucket.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotBackActorBucket#touchRotLimit"), 10));
    end;

    local rotLeftDoorNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.rotLeftDoor#index"));
        if rotLeftDoorNode ~= nil then
        self.rotLeftDoor = {};
        self.rotLeftDoor.node = rotLeftDoorNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotLeftDoor#minRot"));
        self.rotLeftDoor.minRot = {};
        self.rotLeftDoor.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotLeftDoor.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotLeftDoor.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotLeftDoor#maxRot"));
        self.rotLeftDoor.maxRot = {};
        self.rotLeftDoor.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotLeftDoor.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotLeftDoor.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.rotLeftDoor.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotLeftDoor#rotTime"), 2)*1000;
        self.rotLeftDoor.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotLeftDoor#touchRotLimit"), 10));
    end;

    local rotOperatorSeatNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.rotOperatorSeat#index"));
        if rotOperatorSeatNode ~= nil then
        self.rotOperatorSeat = {};
        self.rotOperatorSeat.node = rotOperatorSeatNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotOperatorSeat#minRot"));
        self.rotOperatorSeat.minRot = {};
        self.rotOperatorSeat.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotOperatorSeat.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotOperatorSeat.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotOperatorSeat#maxRot"));
        self.rotOperatorSeat.maxRot = {};
        self.rotOperatorSeat.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotOperatorSeat.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotOperatorSeat.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.rotOperatorSeat.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotOperatorSeat#rotTime"), 2)*1000;
        self.rotOperatorSeat.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotOperatorSeat#touchRotLimit"), 10));
    end;

    local transLeftJackNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.transLeftJack#index"));
        if transLeftJackNode ~= nil then
        self.transLeftJack = {};
        self.transLeftJack.node = transLeftJackNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.transLeftJack#minTrans"));
        self.transLeftJack.minTrans = {};
        self.transLeftJack.minTrans[1] = Utils.getNoNil(x, 0);
        self.transLeftJack.minTrans[2] = Utils.getNoNil(y, 0);
        self.transLeftJack.minTrans[3] = Utils.getNoNil(z, 0);

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.transLeftJack#maxTrans"));
        self.transLeftJack.maxTrans = {};
        self.transLeftJack.maxTrans[1] = Utils.getNoNil(x, 0);
        self.transLeftJack.maxTrans[2] = Utils.getNoNil(y, 0);
        self.transLeftJack.maxTrans[3] = Utils.getNoNil(z, 0);

        self.transLeftJack.transTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.transLeftJack#transTime"), 2)*1000;
        self.transLeftJack.touchTransLimit = Utils.getNoNil(getXMLString(xmlFile, "vehicle.transLeftJack#touchTransLimit"), 10);
    end;

    local transRightJackNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.transRightJack#index"));
        if transRightJackNode ~= nil then
        self.transRightJack = {};
        self.transRightJack.node = transRightJackNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.transRightJack#minTrans"));
        self.transRightJack.minTrans = {};
        self.transRightJack.minTrans[1] = Utils.getNoNil(x, 0);
        self.transRightJack.minTrans[2] = Utils.getNoNil(y, 0);
        self.transRightJack.minTrans[3] = Utils.getNoNil(z, 0);

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.transRightJack#maxTrans"));
        self.transRightJack.maxTrans = {};
        self.transRightJack.maxTrans[1] = Utils.getNoNil(x, 0);
        self.transRightJack.maxTrans[2] = Utils.getNoNil(y, 0);
        self.transRightJack.maxTrans[3] = Utils.getNoNil(z, 0);

        self.transRightJack.transTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.transRightJack#transTime"), 2)*1000;
        self.transRightJack.touchTransLimit = Utils.getNoNil(getXMLString(xmlFile, "vehicle.transRightJack#touchTransLimit"), 10);
    end;


    self.rearWindow = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.rearWindow#index"));
    self.rearWindowClosed = true;

    --Work Camera
    self.workCamera = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.workCamera#index"));
    self.workCameraDriveFOV = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.workCamera#driveFOV"), 60);
    self.workCameraWorkFOV = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.workCamera#workFOV"), 80);

    -- Engine Load
    self.engineUnderLoadEvent = SpecializationUtil.callSpecializationsFunction("engineUnderLoadEvent");
    self.setEngineLoadState = SpecializationUtil.callSpecializationsFunction("setEngineLoadState");
    self.engineUnderLoadEffect = {};
    self.engineUnderLoadEffect.engineIsUnderLoad = false;
    self.engineUnderLoadEffect.motorSoundPitchOffsetNormal = self.motorSoundPitchOffset;
    self.engineUnderLoadEffect.exhaustParticleSystems = {};
    local underLoadExhaustParticleSystemCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.underLoadExhaustParticleSystems#count"), 0);
    for i=1, underLoadExhaustParticleSystemCount do
        local namei = string.format("vehicle.underLoadExhaustParticleSystems.exhaustParticleSystem%d", i);
        Utils.loadParticleSystem(xmlFile, self.engineUnderLoadEffect.exhaustParticleSystems, namei, self.components, false, nil, self.baseDirectory)
    end;
    Utils.setEmittingState(self.engineUnderLoadEffect.exhaustParticleSystems, false);
    self.engineUnderLoadEffect.enabled = getXMLBool(xmlFile, "vehicle.engineUnderLoadEffect#enabled");
    self.engineUnderLoadEffect.enabledDriving = getXMLBool(xmlFile, "vehicle.engineUnderLoadEffect#Driving");
    self.engineUnderLoadEffect.enabledFLBoom = getXMLBool(xmlFile, "vehicle.engineUnderLoadEffect#FLBoom");
    self.engineUnderLoadEffect.enabledFLTool = getXMLBool(xmlFile, "vehicle.engineUnderLoadEffect#FLTool");
    self.engineUnderLoadEffect.enabledBABoom = getXMLBool(xmlFile, "vehicle.engineUnderLoadEffect#BABoom");
    self.engineUnderLoadEffect.enabledBATool = getXMLBool(xmlFile, "vehicle.engineUnderLoadEffect#BATool");
    self.engineUnderLoadEffect.enabledJacks = getXMLBool(xmlFile, "vehicle.engineUnderLoadEffect#Jacks");
    self.engineUnderLoadEffect.pitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.engineUnderLoadEffect#pitchOffset"), 0.05);

    -- HUD
    self.hudOverlay = Overlay:new("hud3CXOverlay", Utils.getFilename("3cx_hud.dds", self.baseDirectory), 0.65, 0.22, 0.33, 0.66);
    self.showHUDInfo = false;

    -- Back Actor Mode
    self.setBackActorJoystickMode = SpecializationUtil.callSpecializationsFunction("setBackActorJoystickMode");
    self.isBackActorMoved = BackHoeDigger.isBackActorMoved;
    self.getPattern = BackHoeDigger.getPattern;
    self.isInBackActorJoystickMode = false;
    self:setBackActorJoystickMode(isInBackActorJoystickMode);
    self.backhoeControlPatterns = {};
    local backhoeControlPatternsCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.backActorControlPatterns#count"), 1);
    for i=1, backhoeControlPatternsCount do
        local patternId = string.format("vehicle.backActorControlPatterns.pattern%d", i);
        local pattern = {};
        pattern.name = Utils.getNoNil(getXMLString(xmlFile, patternId .. "#name"),"");
        pattern.boomUpAction = {};
        pattern.boomDownAction = {};
        pattern.boomLeftAction = {};
        pattern.boomRightAction = {};
        pattern.dipperInAction = {};
        pattern.dipperOutAction = {};
        pattern.bucketInAction = {};
        pattern.bucketOutAction = {};
        pattern.boomUpAction.axis = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".boomUpAction#axis"),"");
        pattern.boomUpAction.axisDirection = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".boomUpAction#axisDirection"),"");
        pattern.boomDownAction.axis = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".boomDownAction#axis"),"");
        pattern.boomDownAction.axisDirection = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".boomDownAction#axisDirection"),"");
        pattern.boomLeftAction.axis = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".boomLeftAction#axis"),"");
        pattern.boomLeftAction.axisDirection = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".boomLeftAction#axisDirection"),"");
        pattern.boomRightAction.axis = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".boomRightAction#axis"),"");
        pattern.boomRightAction.axisDirection = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".boomRightAction#axisDirection"),"");
        pattern.dipperInAction.axis = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".dipperInAction#axis"),"");
        pattern.dipperInAction.axisDirection = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".dipperInAction#axisDirection"),"");
        pattern.dipperOutAction.axis = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".dipperOutAction#axis"),"");
        pattern.dipperOutAction.axisDirection = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".dipperOutAction#axisDirection"),"");
        pattern.bucketInAction.axis = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".bucketInAction#axis"),"");
        pattern.bucketInAction.axisDirection = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".bucketInAction#axisDirection"),"");
        pattern.bucketOutAction.axis = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".bucketOutAction#axis"),"");
        pattern.bucketOutAction.axisDirection = Utils.getNoNil(getXMLString(xmlFile, patternId .. ".bucketOutAction#axisDirection"),"");
        self.backhoeControlPatterns[i] = pattern;
    end;
    self.selectedBackhoeControlPattern = 1;

end;

function BackHoeDigger:delete()
         Utils.deleteParticleSystem(self.engineUnderLoadEffect.exhaustParticleSystems);
         
         if self.hudOverlay ~= nil then
               self.hudOverlay:delete();
         end;
end;

function BackHoeDigger:mouseEvent(posX, posY, isDown, isUp, button)
end;

function BackHoeDigger:keyEvent(unicode, sym, modifier, isDown)
end;

function BackHoeDigger:update(dt)

        self.engineUnderLoadEffect.engineIsUnderLoad = false;

        --if self.isEntered and self.isMotorStarted then    -- not checking for motor started because of bug where animations (sequences only - see dipper and boom rams) continue for short period after engine stops
        if self.isEntered then


        -- Back actor slide
        local slideLeft = InputBinding.isPressed(InputBinding.Backhoe_ACTOR_SLIDE_LEFT);
       	local slideRight = InputBinding.isPressed(InputBinding.Backhoe_ACTOR_SLIDE_RIGHT);
        if self.transBackActor ~= nil and (slideLeft or slideRight) then
		local x, y, z = getTranslation(self.transBackActor.node);
		local trans = {x,y,z};
		local newTrans = Utils.getMovedLimitedValues(trans, self.transBackActor.maxTrans, self.transBackActor.minTrans, 3, self.transBackActor.transTime, dt, not slideLeft);
		setTranslation(self.transBackActor.node, unpack(newTrans));

                self:engineUnderLoadEvent(self.engineUnderLoadEffect.enabledBABoom);
	end;


        -- Back actor dipper
        local dipperOut = self:isBackActorMoved(self:getPattern().dipperOutAction);
       	local dipperIn = self:isBackActorMoved(self:getPattern().dipperInAction);
	if self.rotBackActorDipper ~= nil and (dipperOut or dipperIn) then
	-- arm
                local a, b, c = getRotation(self.rotBackActorDipper.node);
                local rot = {a,b,c};
                local newRot = Utils.getMovedLimitedValues(rot, self.rotBackActorDipper.maxRot, self.rotBackActorDipper.minRot, 3, self.rotBackActorDipper.rotTime, dt, not dipperOut);
                setRotation(self.rotBackActorDipper.node, unpack(newRot));

                self:engineUnderLoadEvent(self.engineUnderLoadEffect.enabledBABoom);
        end;
        -- ram
	if self.dipperRamAnimCharSet ~= nil then
           if self:isBackActorMoved(self:getPattern().dipperOutAction) and self.dipperRamAnimSpeed ~= 1
              and getAnimTrackTime(self.dipperRamAnimCharSet, 0) <= self.dipperRamAnimDuration then
                  self.dipperRamAnimSpeed = 1;
                  setAnimTrackSpeedScale(self.dipperRamAnimCharSet, 0, self.dipperRamAnimSpeed);
           elseif self:isBackActorMoved(self:getPattern().dipperInAction) and self.dipperRamAnimSpeed ~= -1
              and getAnimTrackTime(self.dipperRamAnimCharSet, 0) >= 0 then
                  self.dipperRamAnimSpeed = -1;
                  setAnimTrackSpeedScale(self.dipperRamAnimCharSet, 0, self.dipperRamAnimSpeed);
           elseif self.dipperRamAnimSpeed ~= 0 then
                  self.dipperRamAnimSpeed = 0;
                  setAnimTrackSpeedScale(self.dipperRamAnimCharSet, 0, self.dipperRamAnimSpeed);
           end;
        end;


        -- Back actor tool
	local openBucket = self:isBackActorMoved(self:getPattern().bucketOutAction);
       	local closeBucket = self:isBackActorMoved(self:getPattern().bucketInAction);
	if self.transBackActorBucketRamPin ~= nil and self.rotBackActorBucketQHitch ~= nil and (openBucket or closeBucket) then
	-- ram piston
                local x, y, z = getTranslation(self.transBackActorBucketRamPin.node);
		local trans = {x,y,z};
		local newTrans = Utils.getMovedLimitedValues(trans, self.transBackActorBucketRamPin.maxTrans, self.transBackActorBucketRamPin.minTrans, 3, self.transBackActorBucketRamPin.transTime, dt, not openBucket);
		setTranslation(self.transBackActorBucketRamPin.node, unpack(newTrans));
	-- bucket open close
                local a, b, c = getRotation(self.rotBackActorBucketQHitch.node);
                local rot = {a,b,c};
                local newRot = Utils.getMovedLimitedValues(rot, self.rotBackActorBucketQHitch.maxRot, self.rotBackActorBucketQHitch.minRot, 3, self.rotBackActorBucketQHitch.rotTime, dt, not openBucket);
                setRotation(self.rotBackActorBucketQHitch.node, unpack(newRot));
                
                self:engineUnderLoadEvent(self.engineUnderLoadEffect.enabledBATool);
        end;
        -- bucket rotate
        local rotateClockwise = InputBinding.isPressed(InputBinding.Backhoe_ACTOR_BUCKET_ROTATE_CWISE);
       	local rotateAnticlockwise = InputBinding.isPressed(InputBinding.Backhoe_ACTOR_BUCKET_ROTATE_ACWISE);
        if self.rotBackActorBucket ~= nil and (rotateClockwise or rotateAnticlockwise) then
                local d, e, f = getRotation(self.rotBackActorBucket.node);
                local rot = {d,e,f};
                local newRot = Utils.getMovedLimitedValues(rot, self.rotBackActorBucket.maxRot, self.rotBackActorBucket.minRot, 3, self.rotBackActorBucket.rotTime, dt, not rotateClockwise);
                setRotation(self.rotBackActorBucket.node, unpack(newRot));
                
                self:engineUnderLoadEvent(self.engineUnderLoadEffect.enabledBATool);
        end;


        -- Back actor telescopic extension
        local extendTelescopic = InputBinding.isPressed(InputBinding.Backhoe_ACTOR_TELESCOPIC_EXTEND);
       	local retractTelescopic = InputBinding.isPressed(InputBinding.Backhoe_ACTOR_TELESCOPIC_RETRACT);
	if self.transBackActorTelescopic ~= nil and (extendTelescopic or retractTelescopic) then
		local x, y, z = getTranslation(self.transBackActorTelescopic.node);
		local trans = {x,y,z}; 
		local newTrans = Utils.getMovedLimitedValues(trans, self.transBackActorTelescopic.maxTrans, self.transBackActorTelescopic.minTrans, 3, self.transBackActorTelescopic.transTime, dt, not extendTelescopic);
		setTranslation(self.transBackActorTelescopic.node, unpack(newTrans));

                self:engineUnderLoadEvent(self.engineUnderLoadEffect.enabledBABoom);
	end;


        -- Back actor slew
	local slewRight = self:isBackActorMoved(self:getPattern().boomRightAction);
       	local slewLeft = self:isBackActorMoved(self:getPattern().boomLeftAction);
        if self.rotBackActor ~= nil and (slewRight or slewLeft) then
                local x, y, z = getRotation(self.rotBackActor.node);
                local rot = {x,y,z};
                local newRot = Utils.getMovedLimitedValues(rot, self.rotBackActor.maxRot, self.rotBackActor.minRot, 3, self.rotBackActor.rotTime, dt, not slewRight);
                setRotation(self.rotBackActor.node, unpack(newRot));
                setJointFrame(self.componentJoints[4].jointIndex,0,self.componentJoints[4].jointNode);

                self:engineUnderLoadEvent(self.engineUnderLoadEffect.enabledBABoom);
        end;


        -- Back actor boom
        local lowerBoom = self:isBackActorMoved(self:getPattern().boomDownAction);
       	local raiseBoom = self:isBackActorMoved(self:getPattern().boomUpAction);
        if self.rotBackActorBoom ~= nil and (lowerBoom or raiseBoom) then
        -- boom
                local x, y, z = getRotation(self.rotBackActorBoom.node);
                local rot = {x,y,z};
                local newRot = Utils.getMovedLimitedValues(rot, self.rotBackActorBoom.maxRot, self.rotBackActorBoom.minRot, 3, self.rotBackActorBoom.rotTime, dt, not lowerBoom);
                setRotation(self.rotBackActorBoom.node, unpack(newRot));
                setJointFrame(self.componentJoints[4].jointIndex,0,self.componentJoints[4].jointNode);

                self:engineUnderLoadEvent(self.engineUnderLoadEffect.enabledBABoom);
        end;
        -- ram
	if self.boomRamAnimCharSet ~= nil then
           if self:isBackActorMoved(self:getPattern().boomDownAction) and self.boomRamAnimSpeed ~= 1
              and getAnimTrackTime(self.boomRamAnimCharSet, 0) <= self.boomRamAnimDuration then
                  self.boomRamAnimSpeed = 1;
                  setAnimTrackSpeedScale(self.boomRamAnimCharSet, 0, self.boomRamAnimSpeed);
           elseif self:isBackActorMoved(self:getPattern().boomUpAction) and self.boomRamAnimSpeed ~= -1
              and getAnimTrackTime(self.boomRamAnimCharSet, 0) >= 0 then
                  self.boomRamAnimSpeed = -1;
                  setAnimTrackSpeedScale(self.boomRamAnimCharSet, 0, self.boomRamAnimSpeed);
           elseif self.boomRamAnimSpeed ~= 0 then
                  self.boomRamAnimSpeed = 0;
                  setAnimTrackSpeedScale(self.boomRamAnimCharSet, 0, self.boomRamAnimSpeed);
           end;
        end;


        -- Front loader
        local armMovement, a = InputBinding.getInputAxis(InputBinding["AXIS_BACKHOE_FRONTLOADER_ARM"]);
        local toolMovement, b = InputBinding.getInputAxis(InputBinding["AXIS_BACKHOE_FRONTLOADER_TOOL"]);
        local tool2Movement, c = InputBinding.getInputAxis(InputBinding["AXIS_BACKHOE_FRONTLOADER_TOOL2"]);
        if not self.isInBackActorJoystickMode then
           self:engineUnderLoadEvent(self.engineUnderLoadEffect.enabledFLBoom and armMovement ~= 0);
           self:engineUnderLoadEvent(self.engineUnderLoadEffect.enabledFLTool and (toolMovement ~= 0 or tool2Movement ~= 0));
        end;


        -- Stabiliser jacks
	local leftJackLower = InputBinding.isPressed(InputBinding.Backhoe_LEFT_JACK_LOWER);
       	local leftJackRaise = InputBinding.isPressed(InputBinding.Backhoe_LEFT_JACK_RAISE);
        if self.transLeftJack ~= nil and (leftJackLower or leftJackRaise) then
		local x, y, z = getTranslation(self.transLeftJack.node);
		local trans = {x,y,z};
		local newTrans = Utils.getMovedLimitedValues(trans, self.transLeftJack.maxTrans, self.transLeftJack.minTrans, 3, self.transLeftJack.transTime, dt, not leftJackLower);
		setTranslation(self.transLeftJack.node, unpack(newTrans));
		setJointFrame(self.componentJoints[1].jointIndex,0,self.componentJoints[1].jointNode);

                self:engineUnderLoadEvent(self.engineUnderLoadEffect.enabledJacks);
	end;

	local rightJackLower = InputBinding.isPressed(InputBinding.Backhoe_RIGHT_JACK_LOWER);
       	local rightJackRaise = InputBinding.isPressed(InputBinding.Backhoe_RIGHT_JACK_RAISE);
        if self.transRightJack ~= nil and (rightJackLower or rightJackRaise) then
		local x, y, z = getTranslation(self.transRightJack.node); 
		local trans = {x,y,z}; 
		local newTrans = Utils.getMovedLimitedValues(trans, self.transRightJack.maxTrans, self.transRightJack.minTrans, 3, self.transRightJack.transTime, dt, not rightJackLower);
		setTranslation(self.transRightJack.node, unpack(newTrans));
		setJointFrame(self.componentJoints[2].jointIndex,0,self.componentJoints[2].jointNode);

                self:engineUnderLoadEvent(self.engineUnderLoadEffect.enabledJacks);
	end;
	
	end;


        if self.isEntered then

        -- Engine Load while driving
        local motion, a = InputBinding.getInputAxis(InputBinding["AXIS_MOVE_FORWARD_VEHICLE"]);
        self:engineUnderLoadEvent(self.engineUnderLoadEffect.enabledDriving and motion ~= 0 and not self.isInBackActorJoystickMode);


        -- Door
       	local openDoor = InputBinding.isPressed(InputBinding.Backhoe_DOOR_OPEN);
       	local closeDoor = InputBinding.isPressed(InputBinding.Backhoe_DOOR_CLOSE);
	if self.rotLeftDoor ~= nil and (openDoor or closeDoor) then
                local x, y, z = getRotation(self.rotLeftDoor.node);
                local rot = {x,y,z};
                local newRot = Utils.getMovedLimitedValues(rot, self.rotLeftDoor.maxRot, self.rotLeftDoor.minRot, 3, self.rotLeftDoor.rotTime, dt, not openDoor);
                setRotation(self.rotLeftDoor.node, unpack(newRot));
        end;


        -- Operator platform
	local backActorView = InputBinding.isPressed(InputBinding.Backhoe_SEAT_BACK_ACTOR_VIEW);
       	local drivingView = InputBinding.isPressed(InputBinding.Backhoe_SEAT_DRIVING_VIEW);
        if self.rotOperatorSeat ~= nil and (backActorView or drivingView) then
                local x, y, z = getRotation(self.rotOperatorSeat.node);
                local rot = {x,y,z};
                local newRot = Utils.getMovedLimitedValues(rot, self.rotOperatorSeat.maxRot, self.rotOperatorSeat.minRot, 3, self.rotOperatorSeat.rotTime, dt, not backActorView);
                setRotation(self.rotOperatorSeat.node, unpack(newRot));
                if backActorView then
                       setFovy(self.workCamera, self.workCameraWorkFOV);  -- wider field of vision for working with back actor
                elseif drivingView then
		       setFovy(self.workCamera, self.workCameraDriveFOV);  -- normal field of vision for driving mode
	        end;
	end;


        -- Rear window
        if InputBinding.hasEvent(InputBinding.Backhoe_REAR_WINDOW_TOGGLE) then
                setVisibility(self.rearWindow, not getVisibility(self.rearWindow));
	end;

        end;


	-- Set engine RPM according to load condition
        self:setEngineLoadState();
        
        
        -- HUD panel controls information display toggle
        if InputBinding.hasEvent(InputBinding.Backhoe_HUD_INFO_TOGGLE) then
                self.showHUDInfo = not self.showHUDInfo;
	end;


        -- Back Actor Work Mode
        if InputBinding.hasEvent(InputBinding.Backhoe_BACK_ACTOR_MODE) then
              self:setBackActorJoystickMode(not self.isInBackActorJoystickMode);
        end;


        -- select control pattern
        if InputBinding.hasEvent(InputBinding.Backhoe_BACK_ACTOR_SELECT_PATTERN) then
             if self.selectedBackhoeControlPattern >= table.getn(self.backhoeControlPatterns) then
                self.selectedBackhoeControlPattern = 1;
             else
                self.selectedBackhoeControlPattern = self.selectedBackhoeControlPattern + 1;
             end;
        end;

        -- If camera is switched while in back actor mode, rerun setBackActorJoystickMode to lock new camera
        if InputBinding.hasEvent(InputBinding.CAMERA_SWITCH) then
             self:setBackActorJoystickMode(self.isInBackActorJoystickMode);
        end;


end;

function BackHoeDigger:draw()
         if self.isEntered and not self.showHUDInfo then
            g_currentMission:addHelpButtonText("Display HUD", InputBinding.Backhoe_HUD_INFO_TOGGLE);
         elseif self.isEntered and self.showHUDInfo then
            g_currentMission:addHelpButtonText("Hide HUD", InputBinding.Backhoe_HUD_INFO_TOGGLE);
         end;
         g_currentMission:addExtraPrintText(string.format("Back Actor Control Pattern: %s", self:getPattern().name or ""));
         g_currentMission:addExtraPrintText(string.format("Back Actor Mode: %s", tostring(self.isInBackActorJoystickMode)));
         
         if self.showHUDInfo then
               self.hudOverlay:render();
         end;
end;

function BackHoeDigger:onLeave()
        self:setBackActorJoystickMode(false);
end;

function BackHoeDigger:onEnter()
        self:setBackActorJoystickMode(false);
end;

function BackHoeDigger:validateAttacherJoint(implement, jointDesc, dt)
    return true;
end;

function BackHoeDigger:setBackActorJoystickMode(boolean)
    self.isInBackActorJoystickMode = boolean;
    -- camera
    self.cameras[self.camIndex].isActivated = not self.isInBackActorJoystickMode;
    -- drive
    self.steeringEnabled = not self.isInBackActorJoystickMode;
    -- front loader
    for _, tool in pairs(self.movingTools) do
        if tool.axisActionIndex == InputBinding["AXIS_BACKHOE_FRONTLOADER_ARM"] or
           tool.axisActionIndex == InputBinding["AXIS_BACKHOE_FRONTLOADER_TOOL"] or
           tool.axisActionIndex == InputBinding["AXIS_BACKHOE_FRONTLOADER_TOOL2"]
        then tool.isActive = not self.isInBackActorJoystickMode;
        end;
    end;
end;

function BackHoeDigger:engineUnderLoadEvent(componentEnabledStatus)
    if self.engineUnderLoadEffect.enabled and componentEnabledStatus and self.isMotorStarted then
       self.engineUnderLoadEffect.engineIsUnderLoad = true;
    end;
end;

function BackHoeDigger:setEngineLoadState()
     	if self.engineUnderLoadEffect.engineIsUnderLoad then
		if self.motorSoundPitchOffset == self.engineUnderLoadEffect.motorSoundPitchOffsetNormal then
			self.motorSoundPitchOffset = self.engineUnderLoadEffect.motorSoundPitchOffsetNormal - math.abs(self.engineUnderLoadEffect.pitchOffset);
			Utils.setEmittingState(self.engineUnderLoadEffect.exhaustParticleSystems, true);
		end;
	else
		if self.motorSoundPitchOffset ~= self.engineUnderLoadEffect.motorSoundPitchOffsetNormal then
			self.motorSoundPitchOffset = self.engineUnderLoadEffect.motorSoundPitchOffsetNormal;
			Utils.setEmittingState(self.engineUnderLoadEffect.exhaustParticleSystems, false);
		end;
	end;
end;

function BackHoeDigger:isBackActorMoved(action)
        if action == nil then
           return false;
        end;

        local analogueActivated = false;
        local digitalActivated = false;

        if action.axis ~= "" and action.axisDirection ~= "" then
           local axes = Utils.splitString(",",action.axis);
           local directions = Utils.splitString(",",action.axisDirection);
           if table.getn(axes) == table.getn(directions) then
              for i=1, table.getn(axes) do
                  analogueActivated = (tonumber(directions[i]) == 1 and InputBinding.getAnalogInputAxis(InputBinding[axes[i]]) > 0.005) or (tonumber(directions[i]) == -1 and InputBinding.getAnalogInputAxis(InputBinding[axes[i]]) < -0.005);
                  digitalActivated = (tonumber(directions[i]) == 1 and InputBinding.getDigitalInputAxis(InputBinding[axes[i]]) > 0.005) or (tonumber(directions[i]) == -1 and InputBinding.getDigitalInputAxis(InputBinding[axes[i]]) < -0.005);
              end;
           end;
        end;

        return (self.isInBackActorJoystickMode and analogueActivated) or digitalActivated;
end;

function BackHoeDigger:getPattern()
        return self.backhoeControlPatterns[self.selectedBackhoeControlPattern];
end;
